@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2004, 2005  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2009, 2011  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2001  Internet Software Consortium.
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_tasks.c,v 1.32.18.6 2005/11/30 03:44:39 marka Exp $ */
+/* $Id: t_tasks.c,v 1.49 2011/07/27 07:45:55 marka Exp $ */
 
 #include <config.h>
 
@@ -68,6 +68,7 @@ t1_callback(isc_task_t *task, isc_event_t *event) {
 
 static void
 t1_shutdown(isc_task_t *task, isc_event_t *event) {
+
 	UNUSED(task);
 
 	t_info("shutdown %s\n", (char *)event->ev_arg);
@@ -76,6 +77,7 @@ t1_shutdown(isc_task_t *task, isc_event_t *event) {
 
 static void
 my_tick(isc_task_t *task, isc_event_t *event) {
+
 	UNUSED(task);
 
 	t_info("%s\n", (char *)event->ev_arg);
@@ -378,7 +380,7 @@ static void
 t1(void) {
 	int	result;
 
-	t_assert("tasks", 1, T_REQUIRED, a1);
+	t_assert("tasks", 1, T_REQUIRED, "%s", a1);
 	result = t_tasks1();
 	t_result(result);
 }
@@ -400,7 +402,7 @@ t2_shutdown(isc_task_t *task, isc_event_t *event) {
 
 	isc_result_t	isc_result;
 
-	task = task; /* notused */
+	UNUSED(task);
 
 	if (event->ev_arg != NULL) {
 		isc_task_destroy((isc_task_t**) &event->ev_arg);
@@ -574,7 +576,7 @@ static const char *a2 = "The task subsystem can create ISC_TASKS_MIN tasks";
 
 static void
 t2(void) {
-	t_assert("tasks", 2, T_REQUIRED, a2);
+	t_assert("tasks", 2, T_REQUIRED, "%s", a2);
 
 	if (threaded)
 		t_result(t_tasks2());
@@ -594,7 +596,8 @@ static	int		T3_nprobs;
 
 static void
 t3_sde1(isc_task_t *task, isc_event_t *event) {
-	task = task;
+
+	UNUSED(task);
 
 	if (T3_nevents != T3_NEVENTS) {
 		t_info("Some events were not processed\n");
@@ -612,7 +615,7 @@ t3_sde1(isc_task_t *task, isc_event_t *event) {
 static void
 t3_sde2(isc_task_t *task, isc_event_t *event) {
 
-	task = task;
+	UNUSED(task);
 
 	if (T3_nevents != T3_NEVENTS) {
 		t_info("Some events were not processed\n");
@@ -631,7 +634,7 @@ static void
 t3_event1(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T3_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -654,7 +657,7 @@ t3_event1(isc_task_t *task, isc_event_t *event) {
 
 static void
 t3_event2(isc_task_t *task, isc_event_t *event) {
-	task = task;
+	UNUSED(task);
 
 	++T3_nevents;
 	isc_event_free(&event);
@@ -733,7 +736,7 @@ t_tasks3(void) {
 	if (isc_result != ISC_R_SUCCESS) {
 		t_info("isc_task_create failed %s\n",
 		       isc_result_totext(isc_result));
-		isc_mutex_unlock(&T3_mx);
+		(void) isc_mutex_unlock(&T3_mx);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
@@ -744,6 +747,14 @@ t_tasks3(void) {
 	 */
 	event = isc_event_allocate(mctx, &senders[1], event_type, t3_event1,
 				   NULL, sizeof(*event));
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		(void) isc_mutex_unlock(&T3_mx);
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event);
 
 	/*
@@ -752,6 +763,14 @@ t_tasks3(void) {
 	for (cnt = 0; cnt < T3_NEVENTS; ++cnt) {
 		event = isc_event_allocate(mctx, &senders[1], event_type,
 					   t3_event2, NULL, sizeof(*event));
+		if (event == NULL) {
+			t_info("isc_event_allocate failed\n");
+			(void) isc_mutex_unlock(&T3_mx);
+			isc_task_destroy(&task);
+			isc_taskmgr_destroy(&tmgr);
+			isc_mem_destroy(&mctx);
+			return(T_UNRESOLVED);
+		}
 		isc_task_send(task, &event);
 	}
 
@@ -762,7 +781,7 @@ t_tasks3(void) {
 	if (isc_result != ISC_R_SUCCESS) {
 		t_info("isc_task_send failed %s\n",
 				isc_result_totext(isc_result));
-		isc_mutex_unlock(&T3_mx);
+		(void) isc_mutex_unlock(&T3_mx);
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
@@ -773,7 +792,7 @@ t_tasks3(void) {
 	if (isc_result != ISC_R_SUCCESS) {
 		t_info("isc_task_send failed %s\n",
 				isc_result_totext(isc_result));
-		isc_mutex_unlock(&T3_mx);
+		(void) isc_mutex_unlock(&T3_mx);
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
@@ -788,14 +807,14 @@ t_tasks3(void) {
 	T3_flag = 1;
 	isc_result = isc_condition_signal(&T3_cv);
 	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_task_send failed %s\n",
+		t_info("isc_condition_signal failed %s\n",
 				isc_result_totext(isc_result));
 		++T3_nprobs;
 	}
 
 	isc_result = isc_mutex_unlock(&T3_mx);
 	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_task_send failed %s\n",
+		t_info("isc_mutex_unlock failed %s\n",
 				isc_result_totext(isc_result));
 		++T3_nprobs;
 	}
@@ -810,11 +829,6 @@ t_tasks3(void) {
 		++T3_nfails;
 	}
 
-	if (T3_nevents != T3_nevents) {
-		t_info("T3_nevents == %d, expected 2\n", T3_nevents);
-		++T3_nfails;
-	}
-
 	result = T_UNRESOLVED;
 
 	if (T3_nfails != 0)
@@ -831,7 +845,7 @@ static const char *a3 =	"When isc_task_shutdown() is called, any shutdown "
 			"LIFO order.";
 static void
 t3(void) {
-	t_assert("tasks", 3, T_REQUIRED, a3);
+	t_assert("tasks", 3, T_REQUIRED, "%s", a3);
 
 	if (threaded)
 		t_result(t_tasks3());
@@ -897,7 +911,6 @@ t_tasks4(void) {
 	T4_nfails = 0;
 	T4_flag = 0;
 
-	result = T_UNRESOLVED;
 	event_type = 4;
 
 	workers = 2;
@@ -936,7 +949,7 @@ t_tasks4(void) {
 		t_info("isc_taskmgr_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T4_mx);
-		isc_condition_destroy(&T4_cv);
+		(void) isc_condition_destroy(&T4_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -946,7 +959,7 @@ t_tasks4(void) {
 		t_info("isc_mutex_lock failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T4_mx);
-		isc_condition_destroy(&T4_cv);
+		(void) isc_condition_destroy(&T4_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
@@ -958,7 +971,7 @@ t_tasks4(void) {
 		t_info("isc_task_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T4_mx);
-		isc_condition_destroy(&T4_cv);
+		(void) isc_condition_destroy(&T4_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
@@ -969,6 +982,15 @@ t_tasks4(void) {
 	 */
 	event = isc_event_allocate(mctx, &senders[1], event_type, t4_event1,
 				   NULL, sizeof(*event));
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		DESTROYLOCK(&T4_mx);
+		isc_task_destroy(&task);
+		(void) isc_condition_destroy(&T4_cv);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event);
 
 	isc_task_shutdown(task);
@@ -1002,7 +1024,7 @@ t_tasks4(void) {
 	isc_task_detach(&task);
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
-	isc_condition_destroy(&T4_cv);
+	(void) isc_condition_destroy(&T4_cv);
 	DESTROYLOCK(&T4_mx);
 
 	result = T_UNRESOLVED;
@@ -1021,7 +1043,7 @@ static const char *a4 =
 
 static void
 t4(void) {
-	t_assert("tasks", 4, T_REQUIRED, a4);
+	t_assert("tasks", 4, T_REQUIRED, "%s", a4);
 
 	if (threaded)
 		t_result(t_tasks4());
@@ -1097,7 +1119,6 @@ t_tasks7(void) {
 	T7_sdflag = 0;
 	T7_eflag = 0;
 
-	result = T_UNRESOLVED;
 	event_type = 7;
 
 	workers = 2;
@@ -1136,7 +1157,7 @@ t_tasks7(void) {
 		t_info("isc_taskmgr_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -1146,7 +1167,7 @@ t_tasks7(void) {
 		t_info("isc_mutex_lock failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_FAIL);
@@ -1158,7 +1179,7 @@ t_tasks7(void) {
 		t_info("isc_task_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_FAIL);
@@ -1169,7 +1190,7 @@ t_tasks7(void) {
 		t_info("isc_task_onshutdown returned %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
@@ -1178,6 +1199,15 @@ t_tasks7(void) {
 
 	event = isc_event_allocate(mctx, &senders[1], event_type, t7_event1,
 				   NULL, sizeof(*event));
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		DESTROYLOCK(&T7_mx);
+		(void) isc_condition_destroy(&T7_cv);
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event);
 
 	isc_task_shutdown(task);
@@ -1191,7 +1221,7 @@ t_tasks7(void) {
 			t_info("isc_time_nowplusinterval failed %s\n",
 			       isc_result_totext(isc_result));
 			DESTROYLOCK(&T7_mx);
-			isc_condition_destroy(&T7_cv);
+			(void) isc_condition_destroy(&T7_cv);
 			isc_task_destroy(&task);
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
@@ -1203,7 +1233,7 @@ t_tasks7(void) {
 			t_info("isc_condition_waituntil returned %s\n",
 			       isc_result_totext(isc_result));
 			DESTROYLOCK(&T7_mx);
-			isc_condition_destroy(&T7_cv);
+			(void) isc_condition_destroy(&T7_cv);
 			isc_task_destroy(&task);
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
@@ -1221,7 +1251,7 @@ t_tasks7(void) {
 	isc_task_detach(&task);
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
-	isc_condition_destroy(&T7_cv);
+	(void) isc_condition_destroy(&T7_cv);
 	DESTROYLOCK(&T7_mx);
 
 	result = T_UNRESOLVED;
@@ -1242,7 +1272,7 @@ static const char *a7 =	"A call to isc_task_create() creates a task that can "
 
 static void
 t7(void) {
-	t_assert("tasks", 7, T_REQUIRED, a7);
+	t_assert("tasks", 7, T_REQUIRED, "%s", a7);
 
 	if (threaded)
 		t_result(t_tasks7());
@@ -1274,7 +1304,7 @@ static void
 t10_event1(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T10_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -1309,7 +1339,7 @@ t10_event2(isc_task_t *task, isc_event_t *event) {
 	int	type_match;
 	int	tag_match;
 
-	task = task;
+	UNUSED(task);
 
 	sender_match = 0;
 	type_match = 0;
@@ -1365,7 +1395,7 @@ static void
 t10_sde(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T10_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -1465,7 +1495,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		       isc_result_totext(isc_result));
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1478,7 +1508,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1491,7 +1521,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1502,6 +1532,16 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 	event = isc_event_allocate(mctx, (void *)1, (isc_eventtype_t)T_CONTROL,
 				   t10_event1, NULL, sizeof(*event));
 
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		DESTROYLOCK(&T10_mx);
+		(void) isc_condition_destroy(&T10_cv);
+		++*nprobs;
+		return;
+	}
 	isc_task_send(task, &event);
 
 	/*
@@ -1519,6 +1559,17 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 					    (isc_eventtype_t)(type + type_cnt),
 					    t10_event2, NULL, sizeof(*event));
 
+				if (eventtab[event_cnt] == NULL) {
+					t_info("isc_event_allocate failed\n");
+					isc_task_destroy(&task);
+					isc_taskmgr_destroy(&tmgr);
+					isc_mem_destroy(&mctx);
+					DESTROYLOCK(&T10_mx);
+					(void) isc_condition_destroy(&T10_cv);
+					++*nprobs;
+					return;
+				}
+
 				eventtab[event_cnt]->ev_tag =
 					(void *)((uintptr_t)tag + tag_cnt);
 
@@ -1577,7 +1628,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1610,7 +1661,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
 			DESTROYLOCK(&T10_mx);
-			isc_condition_destroy(&T10_cv);
+			(void) isc_condition_destroy(&T10_cv);
 			++*nprobs;
 			return;
 		}
@@ -1623,7 +1674,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
 			DESTROYLOCK(&T10_mx);
-			isc_condition_destroy(&T10_cv);
+			(void) isc_condition_destroy(&T10_cv);
 			++*nfails;
 			return;
 		}
@@ -1640,7 +1691,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
 	DESTROYLOCK(&T10_mx);
-	isc_condition_destroy(&T10_cv);
+	(void) isc_condition_destroy(&T10_cv);
 
 	if (T_debug)
 		t_info("task processed %d events\n", T10_eventcnt);
@@ -1712,7 +1763,7 @@ static const char *a10 =
 
 static void
 t10(void) {
-	t_assert("tasks", 10, T_REQUIRED, a10);
+	t_assert("tasks", 10, T_REQUIRED, "%s", a10);
 
 	if (threaded)
 		t_result(t_tasks10());
@@ -1732,7 +1783,7 @@ static void
 t11_event1(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T11_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -1857,7 +1908,7 @@ t_tasks11(int purgable) {
 		       isc_result_totext(isc_result));
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T11_mx);
-		isc_condition_destroy(&T11_cv);
+		(void) isc_condition_destroy(&T11_cv);
 		return(T_UNRESOLVED);
 	}
 
@@ -1869,7 +1920,7 @@ t_tasks11(int purgable) {
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T11_mx);
-		isc_condition_destroy(&T11_cv);
+		(void) isc_condition_destroy(&T11_cv);
 		return(T_UNRESOLVED);
 	}
 
@@ -1881,7 +1932,7 @@ t_tasks11(int purgable) {
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T11_mx);
-		isc_condition_destroy(&T11_cv);
+		(void) isc_condition_destroy(&T11_cv);
 		return(T_UNRESOLVED);
 	}
 
@@ -1890,11 +1941,28 @@ t_tasks11(int purgable) {
 	 */
 	event1 = isc_event_allocate(mctx, (void *)1, (isc_eventtype_t)1,
 				    t11_event1, NULL, sizeof(*event1));
-
+	if (event1 == NULL) {
+		t_info("isc_event_allocate failed\n");
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		DESTROYLOCK(&T11_mx);
+		(void) isc_condition_destroy(&T11_cv);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event1);
 
 	event2 = isc_event_allocate(mctx, (void *)1, (isc_eventtype_t)1,
 				    t11_event2, NULL, sizeof(*event2));
+	if (event2 == NULL) {
+		t_info("isc_event_allocate failed\n");
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		DESTROYLOCK(&T11_mx);
+		(void) isc_condition_destroy(&T11_cv);
+		return(T_UNRESOLVED);
+	}
 	event2_clone = event2;
 	if (purgable)
 		event2->ev_attributes &= ~ISC_EVENTATTR_NOPURGE;
@@ -1964,7 +2032,7 @@ t_tasks11(int purgable) {
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
 	DESTROYLOCK(&T11_mx);
-	isc_condition_destroy(&T11_cv);
+	(void) isc_condition_destroy(&T11_cv);
 
 	if (T11_eventcnt != (purgable ? 0 : 1)) {
 		t_info("Event was %s purged\n",
@@ -1989,8 +2057,8 @@ static const char *a11 =
 
 static void
 t11(void) {
-	t_assert("tasks", 11, T_REQUIRED, a11);
-	
+	t_assert("tasks", 11, T_REQUIRED, "%s", a11);
+
 	if (threaded)
 		t_result(t_tasks11(1));
 	else
@@ -2010,7 +2078,7 @@ t_tasks12(void) {
 
 static void
 t12(void) {
-	t_assert("tasks", 12, T_REQUIRED, a12);
+	t_assert("tasks", 12, T_REQUIRED, "%s", a12);
 
 	if (threaded)
 		t_result(t_tasks12());
@@ -2120,7 +2188,7 @@ t_tasks13(void) {
 
 static void
 t13(void) {
-	t_assert("tasks", 13, T_REQUIRED, a13);
+	t_assert("tasks", 13, T_REQUIRED, "%s", a13);
 
 	if (threaded)
 		t_result(t_tasks13());
@@ -2144,7 +2212,7 @@ t14_callback(isc_task_t *task, isc_event_t *event) {
 	int taskno = *(int *)(event->ev_arg);
 
 
-	t_info("task enter %d\n", taskno);	
+	t_info("task enter %d\n", taskno);
 	if (taskno == T14_EXCLTASK) {
 		int	i;
 		t14_exclusiveerror = isc_task_beginexclusive(task);
@@ -2154,7 +2222,7 @@ t14_callback(isc_task_t *task, isc_event_t *event) {
 			t_info("task %d failed to got exclusive access: %d\n",
 				taskno, t14_exclusiveerror);
 		for (i = 0; i < T14_NTASKS; i++) {
-   		        t_info("task %d state %d\n", i , t14_active[i]);
+			t_info("task %d state %d\n", i , t14_active[i]);
 			if (t14_active[i])
 				t14_error++;
 		}
@@ -2272,8 +2340,8 @@ static void
 t14(void) {
 	int	result;
 
-	t_assert("tasks", 14, T_REQUIRED, 
-                 "isc_task_beginexclusive() gets exclusive access");
+	t_assert("tasks", 14, T_REQUIRED, "%s",
+		 "isc_task_beginexclusive() gets exclusive access");
 	result = t_tasks14();
 	t_result(result);
 }
